@@ -25,6 +25,7 @@ package org.kohsuke.stapler;
 
 import net.sf.json.JSON;
 import net.sf.json.JSONArray;
+import net.sf.json.JSONNull;
 import net.sf.json.JSONObject;
 import net.sf.json.util.JSONUtils;
 import org.kohsuke.stapler.export.Flavor;
@@ -79,10 +80,13 @@ public abstract class HttpResponseRenderer {
                     w.print(response);
                 } else
                 if (response instanceof Collection || (response!=null && response.getClass().isArray())) {
-                    JSONArray.fromObject(response).write(w);
+                    JSONArray.fromObject(response, rsp.getJsonConfig()).write(w);
+                } else
+                if (response==null) {
+                    JSONNull.getInstance().write(w);
                 } else {
                     // last fall back
-                    JSONObject.fromObject(response).write(w);
+                    JSONObject.fromObject(response, rsp.getJsonConfig()).write(w);
                 }
                 return true;
             }
@@ -102,7 +106,18 @@ public abstract class HttpResponseRenderer {
             if (response instanceof HttpResponse) {
                 // let the result render the response
                 HttpResponse r = (HttpResponse) response;
-                r.generateResponse(req,rsp,node);
+                try {
+                    r.generateResponse(req,rsp,node);
+                } catch (IOException e) {
+                    if (!handleHttpResponse(req,rsp,node,e))
+                        throw e;
+                } catch (RuntimeException e) {
+                    if (!handleHttpResponse(req,rsp,node,e))
+                        throw e;
+                } catch (ServletException e) {
+                    if (!handleHttpResponse(req,rsp,node,e))
+                        throw e;
+                }
                 return true;
             }
             return false;
