@@ -26,6 +26,7 @@ package org.kohsuke.stapler;
 import net.sf.json.JSON;
 import net.sf.json.JSONArray;
 import net.sf.json.JSONException;
+import net.sf.json.JSONNull;
 import net.sf.json.JSONObject;
 import org.apache.commons.beanutils.BeanUtils;
 import org.apache.commons.beanutils.ConvertUtils;
@@ -530,7 +531,7 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
             Object r = bindInterceptor.onConvert(genericType, type, o);
             if (r!= BindInterceptor.DEFAULT)    return r; // taken over by the listener
 
-            if(o==null) {
+            if(o==null || o instanceof JSONNull) {
                 // this method returns null if the type is not primitive, which works.
                 return ReflectionUtils.getVmDefaultValueFor(type);
             }
@@ -756,7 +757,7 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
                 isSubmission = !getParameterMap().isEmpty();
             }
             
-            if(p==null || p.equals("")) {
+            if(p==null || p.length() == 0) {
                 // no data submitted
                 try {
                     StaplerResponse rsp = Stapler.getCurrentResponse();
@@ -765,15 +766,15 @@ public class RequestImpl extends HttpServletRequestWrapper implements StaplerReq
                     else
                         rsp.sendError(SC_BAD_REQUEST,"Nothing is submitted");
                     rsp.getWriter().close();
-                    throw new Error("This page expects a form submission");
+                    throw new ServletException("This page expects a form submission but had only " + getParameterMap());
                 } catch (IOException e) {
-                    throw new Error(e);
+                    throw new ServletException(e);
                 }
             }
             try {
                 structuredForm = JSONObject.fromObject(p);
             } catch (JSONException e) {
-                throw new JSONException("Failed to parse JSON:"+p,e);
+                throw new ServletException("Failed to parse JSON:" + p, e);
             }
         }
         return structuredForm;
