@@ -24,6 +24,7 @@
 package org.kohsuke.stapler;
 
 import edu.umd.cs.findbugs.annotations.NonNull;
+import net.sf.json.JsonConfig;
 import org.kohsuke.stapler.compression.CompressionFilter;
 import org.kohsuke.stapler.compression.FilterServletOutputStream;
 import org.kohsuke.stapler.export.ExportConfig;
@@ -47,7 +48,7 @@ import java.io.OutputStream;
 import java.io.Writer;
 import java.net.URL;
 import java.net.HttpURLConnection;
-import java.util.zip.GZIPOutputStream;
+import com.jcraft.jzlib.GZIPOutputStream;
 import java.util.Enumeration;
 import java.util.List;
 import java.util.Map;
@@ -69,6 +70,8 @@ public class ResponseImpl extends HttpServletResponseWrapper implements StaplerR
     private OutputMode mode=null;
     private Throwable origin;
 
+    private JsonConfig jsonConfig;
+
     /**
      * {@link ServletOutputStream} or {@link PrintWriter}, set when {@link #mode} is set.
      */
@@ -272,7 +275,7 @@ public class ResponseImpl extends HttpServletResponseWrapper implements StaplerR
         if(acceptEncoding==null || acceptEncoding.indexOf("gzip")==-1)
             return getOutputStream();   // compression not available
 
-        addHeader("Content-Encoding","gzip");
+        setHeader("Content-Encoding","gzip");
         if (CompressionFilter.has(req))
             return getOutputStream(); // CompressionFilter will set up compression. no need to do anything
 
@@ -286,7 +289,7 @@ public class ResponseImpl extends HttpServletResponseWrapper implements StaplerR
         if(acceptEncoding==null || acceptEncoding.indexOf("gzip")==-1)
             return getWriter();   // compression not available
 
-        addHeader("Content-Encoding","gzip");
+        setHeader("Content-Encoding","gzip");
         if (CompressionFilter.has(req))
             return getWriter(); // CompressionFilter will set up compression. no need to do anything
 
@@ -329,6 +332,17 @@ public class ResponseImpl extends HttpServletResponseWrapper implements StaplerR
         return code;
     }
 
+    public void setJsonConfig(JsonConfig config) {
+        jsonConfig = config;
+    }
+
+    public JsonConfig getJsonConfig() {
+        if (jsonConfig == null) {
+            jsonConfig = new JsonConfig();
+        }
+        return jsonConfig;
+    }
+
     private void copyAndClose(InputStream in, OutputStream out) throws IOException {
         IOUtils.copy(in, out);
         IOUtils.closeQuietly(in);
