@@ -3,20 +3,13 @@ package org.kohsuke.stapler.compression;
 import javax.servlet.Filter;
 import javax.servlet.FilterChain;
 import javax.servlet.FilterConfig;
+import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import javax.servlet.ServletRequest;
 import javax.servlet.ServletResponse;
+import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import java.io.IOException;
-import java.io.PrintWriter;
-import java.io.StringWriter;
-import java.text.MessageFormat;
-import java.util.Date;
-import java.util.logging.Level;
-import java.util.logging.Logger;
-import java.util.zip.GZIPOutputStream;
-
-import static org.kohsuke.stapler.Stapler.*;
 
 /**
  * Pimps up {@link HttpServletResponse} so that it understands "Content-Encoding: gzip" and compress the response.
@@ -24,7 +17,7 @@ import static org.kohsuke.stapler.Stapler.*;
  * <p>
  * When exceptions are processed within web applications, different unrelated parts of the webapp can end up calling
  * {@link HttpServletResponse#getOutputStream()}. This fundamentally doesn't work with the notion that the application
- * needs to process "Content-Encoding:gzip" on its own. Such app has to maintain {@link GZIPOutputStream} on its own,
+ * needs to process "Content-Encoding:gzip" on its own. Such app has to maintain a GZIP output stream on its own,
  * since {@link HttpServletResponse} doesn't know that its output is written through a compressed stream.
  *
  * <p>
@@ -40,7 +33,10 @@ import static org.kohsuke.stapler.Stapler.*;
  * @author Kohsuke Kawaguchi
  */
 public class CompressionFilter implements Filter {
+    private ServletContext context;
+
     public void init(FilterConfig filterConfig) throws ServletException {
+        context = filterConfig.getServletContext();
     }
 
     public void doFilter(ServletRequest _req, ServletResponse _rsp, FilterChain filterChain) throws IOException, ServletException {
@@ -50,39 +46,36 @@ public class CompressionFilter implements Filter {
             filterChain.doFilter(_req, rsp);
         } catch (IOException e) {
             if (DISABLED)   throw e;
-            reportException(e,rsp);
+            reportException(e,(HttpServletRequest)_req,rsp);
         } catch (ServletException e) {
             if (DISABLED)   throw e;
-            reportException(e,rsp);
+            reportException(e,(HttpServletRequest)_req,rsp);
+        } catch (RuntimeException e) {
+            if (DISABLED)   throw e;
+            reportException(e,(HttpServletRequest)_req,rsp);
+        } catch (Error e) {
+            if (DISABLED)   throw e;
+            reportException(e,(HttpServletRequest)_req,rsp);
         }
     }
 
-    private void reportException(Exception e, HttpServletResponse rsp) throws IOException {
-        StringWriter sw = new StringWriter();
-        PrintWriter pw = new PrintWriter(sw);
-        e.printStackTrace(pw);
-        pw.close();
-
-        rsp.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
-        rsp.setContentType("text/html");
-        PrintWriter w = rsp.getWriter();
-        String message = e.getMessage();
-        w.print(MessageFormat.format("<html><head><title>Error {0}</title></head>\n" +
-                "<body bgcolor=#fff><h1>Status Code: {0}</h1>Exception: {1}<br>Stacktrace: <pre>{2}</pre><br><hr>\n" +
-                "<i>Generated by Stapler at {3}</i></body></html>",
-                HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
-                message != null ? escape(message) : "?",
-                escape(sw.toString()),
-                new Date().toString()
-        ));
-        w.close();
-
-        LOGGER.log(Level.WARNING, "Untrapped servlet exception", e);
+    private void reportException(Throwable e, HttpServletRequest req, HttpServletResponse rsp) throws IOException, ServletException {
+        getUncaughtExceptionHandler(context).reportException(e, context, req, rsp);
     }
 
     public void destroy() {
     }
 
+    public static void setUncaughtExceptionHandler(ServletContext context, UncaughtExceptionHandler handler) {
+        context.setAttribute(UncaughtExceptionHandler.class.getName(),handler);
+    }
+
+    public static UncaughtExceptionHandler getUncaughtExceptionHandler(ServletContext context) {
+        UncaughtExceptionHandler h = (UncaughtExceptionHandler) context.getAttribute(UncaughtExceptionHandler.class.getName());
+        if (h==null)    h=UncaughtExceptionHandler.DEFAULT;
+        return h;
+    }
+
     /**
      * Is this request already wrapped into {@link CompressionFilter}?
      */
@@ -91,6 +84,4 @@ public class CompressionFilter implements Filter {
     }
     
     public static boolean DISABLED = Boolean.getBoolean(CompressionFilter.class.getName()+".disabled");
-
-    private static final Logger LOGGER = Logger.getLogger("UntrappedServletException");
 }
